function [ Aco, Qp, Ug, Uch ] = esmfsolver_trill( Ps, Cp, Qp, pcoef, T, Coup, bool )
% Solve acoustic equations along the vocal tract for one time step.
% Structures should be sorted according to the following order : 
% 1 = oral tract, 2 = nasal tract, n>2 = various cavities. If they exist: N-1 = subglottal
% tract, N = glottal chink.
% 
% Input arguments :
% 
% Ps : subglottal pressure (in bar)
% Cp : structure containing acoustical parameters of each pipe
%                               .bj : matrix coefficients
%                               .Hj : diagonal matrix coefficients
%                               .Cj : acoustic compliance
%                               .Lj : acoustic mass
%                               .Rj : acoustic resistance
%                               .Gw : total wall conductance
%                               .wc : wall compliance
%                               .wl : wall mass
%                               .Udj : flow source
%                               .aterm : termination area ( = eps if closed pipe)
%                               .Srad : radiation susceptance
%                               .Lg : glottal acoustic mass (only in Cp(1))
% Qp : structure containing derivative and summing terms of each pipe
%                               .Q : derivative term due to acoustic mass
%                               .Qnc : derivative term at coupling point
%                               .Qwc : integration term due to wall compliance
%                               .Qwl : derivative term due to wall mass
%                               .Vc : integration term
%                               .V : integration term
% pcoef : vector containing the 2nd-order and 1st-order terms of the quadratic equation at the glottis
% T : sampling time (in s)
% Coup : structure array containing the coupling relationships between pipes
%                   .child : define the child(ren) of the considered pipe
%                   .parents : define the parent(s) of the considered pipe
%                   .twin : define the twin(s) of the considered pipe
%                   .anabranch : define the anabranch(es) of the considered pipe
% bool : boolean used to determine if vocal folds are in contact. Used only
% for self-oscillating model of vocal folds. Default value is 0 (no
% contact)
%
% Output arguments :
%
% Aco : structure containing acoustic elements of each pipe
%                               .U : volume velocity along each pipe
%                               .P : acoustic pressure
% Qp : structure containing the updated derivative and summing terms (same fields than in input)
% Ug : volume velocity at the glottis
%
% Benjamin Elie, june 2014, last modification, Oct. 2016, to include the
% trachea

if nargin < 7; bool = 0; end;

a_eq = pcoef(1); b_eq = pcoef(2); booltt = pcoef(3); ttpos = pcoef(4); rbtt = pcoef(5); utt = pcoef(6); uttp1 = pcoef(7);
Lgl = Cp(1).Lg+Cp(1).Lj(1);
Npipe = length(Cp);
Nx = zeros(Npipe,1);
Aco(Npipe) = struct('U',[],'P',[]);

for k = 1:Npipe   
    Nx(k) = size(Cp(k).Rj,1);   
end

sgbool = 0;
chbool = 0;
ich = []; for k = 1:length(Coup); if strcmp(Coup(k).Nat,'Chink'); ich = [ich, k]; chbool = 1; end; end;
isg = []; for k = 1:length(Coup); if strcmp(Coup(k).Nat,'subglott'); isg = [isg, k]; sgbool = 1; end; end;
nspipe = chbool+sgbool;
if sgbool
    H1_sg = -(2/T*Cp(isg).Lj(1)+Cp(isg).Rj(1)+Cp(isg).bj(1)+Cp(isg).Rcm(1));
    Hsg = -(2/T*Cp(isg).Lj(Nx(isg))+Cp(isg).Rj(Nx(isg))+Cp(isg).bj(Nx(isg))+Cp(isg).Rcp(Nx(isg)));
end
    
if chbool
    Lch = Cp(ich).Lj(1);  
    Hch = -(Cp(1).bj(1)+Cp(1).Rj(1)+2*Cp(1).Lj(1)/T+Cp(1).Rcm(1));      
    H1_ch = -(Cp(ich).bj(1)+Cp(1).bj(1)+Cp(ich).Rj(1)+Cp(1).Rj(1)+2*(Cp(ich).Lj(1)+Cp(1).Lj(1))/T);
    if sgbool        
        Fch = Cp(1).bj(1)*(Cp(1).Udj(1)+Qp(1).V(1))-Qp(ich).Q(1)+Cp(ich).Ns(1);
    else 
        Fch = Cp(1).bj(1)*(Cp(1).Udj(1)+Qp(1).V(1))-Ps-Qp(ich).Q(1)+Cp(ich).Ns(1);
    end
else
    Uch = 0;
end
Ft = zeros(sum(Nx+1)-chbool,1);
Wont = zeros(sum(Nx+1)-chbool);
W = [];

for k = 1:Npipe
    if k == 1    
        Fj = zeros(Nx(k)+1,1);
        Qp(k).V(1:Nx(k)) = Qp(k).Vc-Cp(k).Gw.*(Qp(k).Qwl-Qp(k).Qwc);
        Fj(2:Nx(k)) = -Cp(k).bj(1:end-2).*(Cp(k).Udj(1:end-1)+Qp(k).V(1:end-2))+Cp(k).bj(2:end-1).*(Cp(k).Udj(2:end)+Qp(k).V(2:end-1))-Qp(k).Q(2:end-1)+Cp(k).Ns(2:Nx(k))-Cp(k).Ns(3:Nx(k)+1);
        Fj(Nx(k)+1) = -Cp(k).bj(Nx(k))*(Cp(k).Udj(Nx(k))+Qp(k).V(Nx(k)))+Cp(k).bj(Nx(k)+1)*Qp(k).V(Nx(k)+1)-Qp(k).Q(Nx(k)+1)+Cp(k).Ns(Nx(k)+1);
        Cp(1).Hj(1) = -b_eq-Cp(1).bj(1);
        if sgbool
            Fj(1) = Cp(k).bj(1)*(Cp(k).Udj(1)+Qp(k).V(1))-Qp(k).Q(1)+Cp(k).Ns(1)-Cp(k).Ns(2);%+a_eq*Ugim1^2;
        else
            Fj(1) = Cp(k).bj(1)*(Cp(k).Udj(1)+Qp(k).V(1))-Ps-Qp(k).Q(1)+Cp(k).Ns(1)-Cp(k).Ns(2); 
        end  
        Cp(1).Hj(ttpos) = Cp(1).Hj(ttpos)-rbtt/2*abs(utt);
        Cp(1).Hj(ttpos+1) = Cp(1).Hj(ttpos+1)+rbtt/2*abs(uttp1);
%         Fj(ttpos) = Fj(ttpos)+rbtt/2*utt^2;
%         Fj(ttpos+1) = Fj(ttpos+1)-rbtt/2*uttp1^2;
%         a_eq = 0;
    else       
        mainpipe = Coup(k).parents(1,1); ncoup = Coup(k).parents(2,1);
        if strcmp(Coup(k).Nat,'Chink')   
            Cp(k).Hj(1) = H1_ch;
            Fj = Fch;   
            Wont(1,end) = Hch;
            Wont(2,end) = Cp(1).bj(1);
            Wont(end,2) = Cp(1).bj(1);
            Wont(end,1) = Hch;                    
        elseif strcmp(Coup(k).Nat,'subglott')    
            Fj = zeros(Nx(k)+1,1);
            Qp(k).V(1:Nx(k)) = Qp(k).Vc-Cp(k).Gw.*(Qp(k).Qwl-Qp(k).Qwc);
            Fj(2:Nx(k)) = -Cp(k).bj(1:end-2).*(Cp(k).Udj(1:end-1)+Qp(k).V(1:end-2))+Cp(k).bj(2:end-1).*(Cp(k).Udj(2:end)+Qp(k).V(2:end-1))-Qp(k).Q(2:end-1)+Cp(k).Ns(2:Nx(k))-Cp(k).Ns(3:Nx(k)+1);
            Cp(k).Hj(1) = H1_sg;
            Cp(isg).Hj(end) = -1;
            Fj(1) = Cp(k).bj(1)*(Cp(k).Udj(1)+Qp(k).V(1))-Ps-Qp(k).Q(1)+Cp(k).Ns(1)-Cp(k).Ns(2);
            Fj(end) = -Cp(k).bj(Nx(k))*(Cp(k).Udj(Nx(k))+Qp(k).V(Nx(k)))-Qp(k).Q(Nx(k)+1);
            Wont(1,end-chbool) = 1;
            Wont(end-nspipe,1) = Cp(isg).bj(Nx(k));
            Wont(end-chbool,1) = Hsg;
            if chbool 
                Wont(end-2,end) = Cp(isg).bj(Nx(k));
                Wont(end-1,end) = Hsg;
                Wont(end,end-1) = 1;
            end
        else    
            Fj = zeros(Nx(k)+1,1);
            Qp(k).V(1:Nx(k)) = Qp(k).Vc-Cp(k).Gw.*(Qp(k).Qwl-Qp(k).Qwc);
            Fj(2:Nx(k)) = -Cp(k).bj(1:end-2).*(Cp(k).Udj(1:end-1)+Qp(k).V(1:end-2))+Cp(k).bj(2:end-1).*(Cp(k).Udj(2:end)+Qp(k).V(2:end-1))-Qp(k).Q(2:end-1)+Cp(k).Ns(2:Nx(k))-Cp(k).Ns(3:Nx(k)+1);
            Fj(Nx(k)+1) = -Cp(k).bj(Nx(k))*(Cp(k).Udj(Nx(k))+Qp(k).V(Nx(k)))+Cp(k).bj(Nx(k)+1)*Qp(k).V(Nx(k)+1)-Qp(k).Q(Nx(k)+1)+Cp(k).Ns(Nx(k)+1);
            Fj(1) = Cp(k).bj(1)*(Cp(k).Udj(1)+Qp(k).V(1))-Cp(mainpipe).bj(ncoup)*(Cp(mainpipe).Udj(ncoup)+Qp(mainpipe).V(ncoup))-Qp(k).Q(1)+Cp(k).Ns(1)-Cp(k).Ns(2);
            Cp(k).Hj(1) = -(2*(Cp(k).Lj(1)+Cp(mainpipe).Lj(ncoup))/T+Cp(k).Rj(1)+Cp(k).Rcm(1)+Cp(mainpipe).Rj(ncoup)+Cp(mainpipe).Rcp(ncoup)+Cp(k).bj(1)+Cp(mainpipe).bj(ncoup));
            Hnc = -(Cp(mainpipe).bj(ncoup)+Cp(mainpipe).Rj(ncoup)+2*Cp(mainpipe).Lj(ncoup)/T+Cp(mainpipe).Rcp(ncoup));
            Wont(ncoup+1+(mainpipe-1)*sum(Nx(1:mainpipe)+1),sum(Nx(1:k-1)+1)+1) = Hnc;
            Wont(sum(Nx(1:k-1)+1)+1,ncoup+1+(mainpipe-1)*sum(Nx(1:mainpipe)+1)) = Hnc;
            Wont(sum(Nx(1:k-1)+1)+1,ncoup+(mainpipe-1)*sum(Nx(1:mainpipe)+1)) = Cp(mainpipe).bj(ncoup);
            Wont(ncoup+(mainpipe-1)*sum(Nx(1:mainpipe)+1),sum(Nx(1:k-1)+1)+1) = Cp(mainpipe).bj(ncoup);
            if ~isempty(Coup(k).twin); Wont(sum(Nx(1:k-1)+1)+1,sum(Nx(1:Coup(k).twin-1)+1)+1) = Hnc; end;
            if size(Coup(k).parents,2) > 1                
                NN = Nx(k); brg = Coup(k).parents(2,2); mainpipe = Coup(k).parents(1,2);
                Fj(end) = -Cp(k).bj(NN)*(Cp(k).Udj(NN)+Qp(k).V(NN))+Cp(mainpipe).bj(brg+1)*(Cp(mainpipe).Udj(brg+1)+Qp(mainpipe).V(brg+1))-Qp(k).Q(NN+1)+Cp(k).Ns(Nx(k)+1);
                Cp(k).Hj(NN+1) =  -(Cp(k).bj(NN)+Cp(mainpipe).bj(brg+1)+2/T*(Cp(k).Lj(NN)+Cp(mainpipe).Lj(brg+1))+Cp(k).Rj(NN)+Cp(k).Rcp(NN)+Cp(mainpipe).Rj(brg+1)+Cp(mainpipe).Rcp(brg+1));
                Hcp1 = -(Cp(mainpipe).bj(brg+1)+Cp(mainpipe).Rj(brg+1)+Cp(mainpipe).Rcp(brg+1)+2*Cp(mainpipe).Lj(brg+1)/T);
                Wont(brg+1+(mainpipe-1)*sum(Nx(1:mainpipe)+1),sum(Nx(1:k-1)+1)+NN+1) = Hcp1;
                Wont(sum(Nx(1:k-1)+1)+1+NN,brg+1+(mainpipe-1)*sum(Nx(1:mainpipe)+1)) = Hcp1;
                Wont(sum(Nx(1:k-1)+1)+1+NN,brg+2+(mainpipe-1)*sum(Nx(1:mainpipe)+1)) = Cp(mainpipe).bj(brg+1);
                Wont(brg+2+(mainpipe-1)*sum(Nx(1:mainpipe)+1),sum(Nx(1:k-1)+1)+NN+1) = Cp(mainpipe).bj(brg+1);                
            end        
        end                      
    end
 
    if strcmp(Coup(k).Nat,'Chink')          
        Ft(end) = Fj;
        Wmat = Cp(k).Hj(1);
        W = [ W,zeros(size(W,1),size(Wmat,2));zeros(size(Wmat,1),size(W,2)),Wmat ];        
    else    
        Ft(1+sum(Nx(1:k-1)+1):sum(Nx(1:k-1)+1)+length(Fj)) = Fj(:);        
        Wmat = diag(Cp(k).Hj);
        bd = diag(Cp(k).bj(1:end-1));
        Wmat(1:end-1,2:end) = Wmat(1:end-1,2:end)+bd;
        Wmat(2:end,1:end-1) = Wmat(2:end,1:end-1)+bd;
        if strcmp(Coup(k).Nat,'subglott')
            Wmat(end-1,end) = 0;
        end
        W = [ W,zeros(size(W,1),size(Wmat,2));zeros(size(Wmat,1),size(W,2)),Wmat ];        
    end

end    
Wont = Wont+W;
% if booltt;  disp('Tongue contact'); end;
if bool
	Ug = 0; % if contact, airflow is null
    % To be used with a self-oscillating model of the vocal folds
    Ft2 = Ft(2:end);  
    Wont2 = Wont(2:end,2:end);
    if booltt % tongue-palate contact     
%         save all;
%         stopstop
        Ft2(ttpos-1:ttpos) = [];
        Wont2(ttpos-1:ttpos,:) = [];
        Wont2(:,ttpos-1:ttpos) = [];
    end  
    Ut = Wont2\Ft2;  Ut = Ut(:);
%     if booltt; Ut = [Ut(1:ttpos-2);0;0;Ut(ttpos:end)]; end;
    if booltt; Ut = [Ut(1:ttpos-2);0; 0;Ut(ttpos-1:end)]; end;
    if chbool; Uch = Ut(end); end;
    if sgbool; po = Ut(end-chbool); end;
else    
    Ft2 = Ft;
    Wont2 = Wont;
    if booltt % tongue-palate contact        
        Ft2(ttpos:ttpos+1) = [];
        Wont2(ttpos:ttpos+1,:) = [];
        Wont2(:,ttpos:ttpos+1) = [];
    end  
    GSF = Wont2\Ft2; % rearrange the system to solve Ug
    F11 = GSF(1); 
    GSi = Wont2\eye(size(Wont2));
    aug = -a_eq*GSi(1,1);        
    if (aug~=0 && ~isinf(aug) && ~isnan(aug));  asolug = solve_pol_2(aug,1,-F11); else asolug = F11; end; % Solve quadratic equation of Ug
    Ug = max(asolug,0); % If the solution is negative, keep 0.
    Ft2 = Ft2(2:end);
    if Ug ~= 0
        Ft2(1) = Ft2(1)-Cp(1).bj(1)*Ug; % Update the linear system by removing the Ug coefficients ....
        if chbool; Ft2(end) = Ft2(end) - Hch*Ug;  end; % at the chink
        if sgbool; Ft2(end-nspipe) = Ft2(end-nspipe)-Cp(isg).bj(Nx(isg))*Ug; Ft2(end-chbool) = Ft2(end-chbool)-Hsg*Ug; end; % at the subglottal connection
    end
    Wont2 = Wont2(2:end,2:end); % remove the first line and the first column of the matrix of the linear coefficients
    Ut = Wont2\Ft2;  % solve remaining system; 
%     if booltt; Ut = [Ut(1:ttpos-2);0;0;Ut(ttpos:end)]; end;
    if booltt; Ut = [Ut(1:ttpos-2);0; 0;Ut(ttpos-1:end)]; end;
    if chbool; Uch = Ut(end); end; % get the flow through the chink
    if sgbool; po = Ut(end-chbool); end; % get the pressure at the subglottal connection
end

for k = 1:Npipe % Store U values in different pipes
    if k == 1          
        Aco(1).U(1) = Ug;
        Aco(k).U(2:Nx(1)+1) = Ut(1:Nx(1));
    else        
        if strcmp(Coup(k).Nat,'Chink')           
            Aco(k).U = Uch;            
        elseif strcmp(Coup(k).Nat,'subglott')  
            Aco(k).U = Ut(sum(Nx(1:k-1))+k-1:sum(Nx(1:k))+k-2);            
            Aco(k).U(Nx(k)+1) = Ug+Uch;           
        else            
            Aco(k).U = Ut(sum(Nx(1:k-1))+k-1:sum(Nx(1:k))+k-1);            
        end    
    end    
	Aco(k).U = Aco(k).U(:);    
end

for k = 1:Npipe  
    ch = [];
    child = Coup(k).child;
    if ~isempty(child); pos_ch = child(2,:); ch = child(1,:); end;
    if k == 1                
        Qp(1).Q(1) = 4*Lgl/T*Ug+4/T*(Cp(1).Lj(1))*Uch-Qp(1).Q(1);         
    else          
        mainpipe = Coup(k).parents(1,1); ncoup = Coup(k).parents(2,1);        
        if strcmp(Coup(k).Nat,'Chink')        
            Qp(k).Q(1) = 4*(Lch+Cp(1).Lj(1))/T*Uch+4/T*Cp(1).Lj(1)*Ug-Qp(k).Q(1); 
            Pt = Cp(k).bj(1).*(Ug+Uch-Aco(1).U(2)+Cp(k).Udj+Qp(k).V(1:end-1));
            Aco(k).P = Pt(:);
            Qp(k).Vc = 4/T*Cp(k).Cj.*Aco(k).P-Qp(k).Vc;
            u3 = Cp(k).Gw.*(Pt+Qp(k).Qwl-Qp(k).Qwc);
            Qp(k).Qwc = 2*Cp(k).wc.*u3+Qp(k).Qwc;
            Qp(k).Qwl = 2*Cp(k).wl.*u3-Qp(k).Qwl;            
        else        
            if ~isempty(Coup(k).twin)
                Qp(k).Q(1) = 4*(Cp(k).Lj(1)+Cp(mainpipe).Lj(ncoup))/T*Aco(k).U(1)+4*Cp(mainpipe).Lj(ncoup)/T*(Aco(mainpipe).U(ncoup+1)+Aco(Coup(k).twin).U(1))-Qp(k).Q(1);                
            elseif strcmp(Coup(k).Nat,'subglott')  
                Qp(k).Q(1) = 4*(Cp(k).Lj(1))/T*Aco(k).U(1)-Qp(k).Q(1);  
            else
                Qp(k).Q(1) = 4*(Cp(k).Lj(1)+Cp(mainpipe).Lj(ncoup))/T*Aco(k).U(1)+4*Cp(mainpipe).Lj(ncoup)/T*Aco(mainpipe).U(ncoup+1)-Qp(k).Q(1);                
            end        
        end    
    end

    if ~strcmp(Coup(k).Nat,'Chink')   
        Pt = Cp(k).bj(1:end-1).*(Aco(k).U(1:end-1)-Aco(k).U(2:end)+Cp(k).Udj+Qp(k).V(1:end-1));
        if k == 1; Pt(1) = Cp(k).bj(1).*(Ug+Uch-Aco(k).U(2)+Cp(k).Udj(1)+Qp(k).V(1));  end;
        Qp(k).Q(2:end-1) = 4*(Cp(k).Lj(1:end-1)+Cp(k).Lj(2:end))/T.*Aco(k).U(2:end-1)-Qp(k).Q(2:end-1);        
        if ~isempty(ch) 
            for Kq = 1:length(unique(pos_ch))                
                ncoup = pos_ch(Kq);
                Qp(k).Qnc(Kq) = 4*(Cp(k).Lj(ncoup)+Cp(k).Lj(ncoup+1))/T*Aco(k).U(ncoup+1)+4/T*Cp(k).Lj(ncoup)*sum(arrayfun(@(x) sum(x.U(1)), Aco(ch(pos_ch == ncoup))))-Qp(k).Qnc(Kq);
                Qp(k).Q(ncoup+1) = Qp(k).Qnc(Kq);
                Pt(ncoup) = Cp(k).bj(ncoup).*(Aco(k).U(ncoup)-Aco(k).U(ncoup+1)-sum(arrayfun(@(x) sum(x.U(1)), Aco(ch(pos_ch == ncoup))))+Cp(k).Udj(ncoup)+Qp(k).V(ncoup));                
            end            
        end
        
        if ~isempty(Coup(k).anabranch)            
            Npt = Coup(k).anabranch; brg = Coup(Npt).parents(2,2);
            Qp(k).Qnc(end) = 4*(Cp(k).Lj(brg)+Cp(k).Lj(brg+1))/T*Aco(k).U(brg+1)+4/T*Cp(k).Lj(brg+1)*Aco(Npt).U(end)-Qp(k).Qnc(end);
            Qp(k).Q(brg+1) = Qp(k).Qnc(end);
            Pt(brg+1) = Cp(k).bj(brg+1).*(Aco(k).U(brg+1)-Aco(k).U(brg+2)+Aco(Npt).U(end)+Cp(k).Udj(brg+1)+Qp(k).V(brg+1));            
        end
        
        Aco(k).P(1:Nx(k)) = Pt(:);
        Aco(k).P = Aco(k).P(:);
        Qp(k).Vc = 4/T*Cp(k).Cj.*Aco(k).P(1:Nx(k))-Qp(k).Vc;
        u3 = Cp(k).Gw.*(Pt+Qp(k).Qwl-Qp(k).Qwc);
        Qp(k).Qwc = 2*Cp(k).wc.*u3+Qp(k).Qwc;
        Qp(k).Qwl = 2*Cp(k).wl.*u3-Qp(k).Qwl;        
        
        if k == 1; Qp(k).Q(Nx(k)+1) = 4*Cp(k).Lj(end)/T*Aco(k).U(Nx(k)+1)-Qp(k).Q(Nx(k)+1); end;
        if k~=1            
            if size(Coup(k).parents,2) > 1                
                brg = Coup(k).parents(2,2); mainpipe = Coup(k).parents(1,2);
                Qp(k).Q(Nx(k)+1) = 4/T*((Cp(k).Lj(end)+Cp(mainpipe).Lj(brg+1))*Aco(k).U(Nx(k)+1)+Cp(mainpipe).Lj(brg+1)*Aco(mainpipe).U(brg+1))-Qp(k).Q(Nx(k)+1);                
            else                
                Qp(k).Q(Nx(k)+1) = 4*Cp(k).Lj(end)/T*Aco(k).U(Nx(k)+1)-Qp(k).Q(Nx(k)+1);                
            end            
        end  
        if ~strcmp(Coup(k).Nat,'subglott')
            Aco(k).P(Nx(k)+1) = Cp(k).bj(Nx(k)+1)*(Aco(k).U(Nx(k)+1)+Qp(k).V(Nx(k)+1));
        else
            Aco(k).P(Nx(k)+1) = po;
        end
        Qp(k).V(Nx(k)+1) = -2*Cp(k).Srad*sqrt(Cp(k).aterm)*Aco(k).P(Nx(k)+1)+Qp(k).V(Nx(k)+1);        
    else        
        Aco(k).P = Aco(1).P(1);        
    end    
end
end

function x = solve_pol_2(a,b,c)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Solve the equation a*x^2+b*x+c=0 and return the greater solution.
% 
% Christophe Varray, Mars-Juillet 2013
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% disp(['a = ',num2str(a)]);
% disp(['b = ',num2str(b)]);
% disp(['c = ',num2str(c)]);

delta = b*b-4*a*c;

if delta>=0
    
    x1 = (-b+sqrt(delta))/(2*a);
    x2 = (-b-sqrt(delta))/(2*a);
    
else
    
    error('no real solution for this equation');
    
end

x = max(x1,x2);

end